# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2011 Gábor Dukai <gdukai@gmail.com>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv

class create_wave(osv.osv_memory):
    _name = 'stock.create.wave'

    def _get_products_total(self, cr, uid, context):
        return self._get_defaults(cr, uid, 'total', context)

    def _get_products_assigned(self, cr, uid, context):
        return self._get_defaults(cr, uid, 'assigned', context)

    def _get_products_other(self, cr, uid, context):
        return self._get_defaults(cr, uid, 'other', context)

    def _get_defaults(self, cr, uid, field, context):
        wave_obj = self.pool.get('stock.wave')
        (assigned_lines, other_lines) \
            = wave_obj.generate_picking_lines(cr, uid, context['active_ids'])
        if field == 'total':
            return len(assigned_lines) + len(other_lines)
        elif field == 'assigned':
            return len(assigned_lines)
        elif field == 'other':
            return len(other_lines)

    _columns = {
        'products_total': fields.integer('All Products', readonly=True),
        'products_assigned': fields.integer('Assigned Products',
            readonly=True),
        'products_other': fields.integer('Other Products',
            readonly=True),
    }

    _defaults = {
        'products_total': _get_products_total,
        'products_assigned': _get_products_assigned,
        'products_other': _get_products_other,
    }

    def action_create(self, cr, uid, ids, context=None):
        wave_obj = self.pool.get('stock.wave')
        wpick_ids \
            = wave_obj.create_from_picking(cr, uid, context['active_ids'])
        if wpick_ids:
            return {
                'view_type': 'form',
                'view_mode': 'tree,form',
                'res_id': wpick_ids,
                'domain': [('id', 'in', wpick_ids)],
                'res_model': 'stock.wave.picking',
                'type': 'ir.actions.act_window',
            }

create_wave()