# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    Copyright (C) 2011 Gábor Dukai <gdukai@gmail.com>
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import netsvc
from tools.misc import UpdateableStr, UpdateableDict
import pooler

import wizard
import tools
from tools.translate import _

_moves_arch = UpdateableStr()
_moves_fields = UpdateableDict()

_moves_arch_end = '''<?xml version="1.0"?>
<form string="Picking result">
    <label string="The picking has been successfully made!" colspan="4"/>
    <field name="wave_pick_notification" colspan="4" nolabel="1"/>
</form>'''
_moves_fields_end = {
    'wave_pick_notification': {'string': 'Wave Pickings' , 'type':'text',
        'readonly': True}
}

def make_default(val):
    def fct(uid, data, state):
        return val
    return fct

def _to_xml(s):
    return (s or '').replace('&','&amp;').replace('<','&lt;').replace('>','&gt;')

def _get_moves(self, cr, uid, data, context):
    wpick_obj = pooler.get_pool(cr.dbname).get('stock.wave.picking')
    wpick = wpick_obj.browse(cr, uid, [data['id']], context)[0]
    res = {}

    _moves_fields.clear()
    _moves_arch_lst = ['<?xml version="1.0"?>', '<form string="Make packing">']

    for line in wpick.line_ids:
        for m in line.move_ids:
            if m.state in ('done', 'cancel'):
                continue
            quantity = m.product_qty
            if m.state != 'assigned':
                quantity = 0
            _moves_arch_lst.append('<field name="move%s" />' % (m.id,))
            _moves_fields['move%s' % m.id] = {
                'string': _to_xml(m.name + ' : ' + m.picking_id.name[0] \
                    + m.picking_id.name[-3:]),
                'type' : 'float', 'required' : True,
                'default' : make_default(quantity)
            }
            _moves_arch_lst.append('<newline/>')
            res.setdefault('moves', []).append(m.id)
    _moves_arch_lst.append('</form>')
    _moves_arch.string = '\n'.join(_moves_arch_lst)
    return res

def _do_split(self, cr, uid, data, context):
    pool = pooler.get_pool(cr.dbname)
    move_obj = pool.get('stock.move')
    pick_obj = pool.get('stock.picking')
    wpick_obj = pool.get('stock.wave.picking')
    wave_obj = pool.get('stock.wave')
    new_pickings = {}
    old_pickings = set()

    complete, too_many, too_few_remaining, too_few_done = [], [], [], []
    pool = pooler.get_pool(cr.dbname)
    for move in move_obj.browse(cr, uid, data['form'].get('moves',[])):
        if move.product_qty == data['form']['move%s' % move.id]:
            complete.append(move)
        elif move.product_qty > data['form']['move%s' % move.id]:
            too_few_remaining.append(move)
        else:
            too_many.append(move)
        old_pickings.add(move.picking_id.id)
    for move in too_few_remaining:
        if move.picking_id.id in new_pickings:
            new_picking = new_pickings[move.picking_id.id]
        elif move.picking_id.backorder_id \
            and move.picking_id.backorder_id.state == 'draft':
            new_picking = move.picking_id.backorder_id.id
        else:
            new_picking = None
        if not new_picking:
            new_picking = pick_obj.copy(cr, uid, move.picking_id.id, {
                'name': pool.get('ir.sequence').get(cr, uid, 'stock.picking'),
                'move_lines' : [],
                'state': 'draft',
            })
        new_pickings[move.picking_id.id] = new_picking
        if data['form']['move%s' % move.id]:
            new_move_id = move_obj.copy(cr, uid, move.id, {
                'product_qty' : data['form']['move%s' % move.id],
                'product_uos_qty':data['form']['move%s' % move.id],
                'picking_id' : new_picking,
                'state': 'assigned',
                'move_dest_id': False,
                'price_unit': move.price_unit,
            })
            too_few_done.append(move_obj.browse(cr, uid, new_move_id))
        move_obj.write(cr, uid, [move.id], {
            'product_qty' : move.product_qty - data['form']['move%s' % move.id],
            'product_uos_qty': move.product_qty - data['form']['move%s' % move.id],
        })
    for c in complete:
        if c.picking_id.id in new_pickings:
            move_obj.write(cr, uid, [c.id], {
                'picking_id': new_pickings[c.picking_id.id]})
    for move in too_many:
        if move.picking_id.id in new_pickings:
            move_obj.write(cr, uid, [move.id], {
                'product_qty' : data['form']['move%s' % move.id],
                'product_uos_qty': data['form']['move%s' % move.id],
                'picking_id': new_pickings[move.picking_id.id],
            })
        else:
            move_obj.write(cr, uid, [move.id], {
                'product_qty': data['form']['move%s' % move.id],
                'product_uos_qty': data['form']['move%s' % move.id]
            })
    wf_service = netsvc.LocalService("workflow")
    for move in complete + too_many + too_few_done:
        if move.state == 'draft':
            move_obj.action_confirm(cr, uid, [move.id])
        if move.state in ['draft', 'confirmed', 'waiting']:
            move_obj.force_assign(cr, uid, [move.id])
        move_obj.action_done(cr, uid, [move.id])
    for old_picking in old_pickings:
        if old_picking in new_pickings:
            move_ids = move_obj.search(cr, uid, [
                ('picking_id', '=', old_picking),
                ('state', 'in', ['done', 'cancel']),
            ])
            move_obj.write(cr, uid, move_ids, {
                'picking_id': new_pickings[old_picking]})
            d = {
                'backorder_id': new_pickings[old_picking],
                'shipment_id': False,
                'move_lines': [],
            }
            pick_obj.write(cr, uid, [old_picking], d)
            wf_service.trg_write(uid, 'stock.picking', old_picking, cr)
            pick_obj.write(cr, uid, [new_pickings[old_picking]],
                {'move_lines': []})
    wpick_obj.write(cr, uid, [data['id']], {'state': 'done'})
    wpick = wpick_obj.browse(cr, uid, data['id'])
    wpicks_remaining = 0
    all_new_pickings = {}
    all_standalone_pickings = set()
    for wpick2 in wpick.wave_id.wave_picking_ids:
        if wpick2.state == 'in_progress':
            wpicks_remaining += 1
        for line in wpick2.line_ids:
            for move in line.move_ids:
                if move.picking_id.backorder_id:
                    all_new_pickings[move.picking_id.id] \
                        = move.picking_id.backorder_id.id
                else:
                    all_standalone_pickings.add(move.picking_id.id)
    if wpicks_remaining:
        all_new_pickings = False
    else:
        for pick_id in list(all_standalone_pickings) + all_new_pickings.values():
            wf_service.trg_validate(uid, 'stock.picking', pick_id,
                'button_confirm', cr)
            wf_service.trg_validate(uid, 'stock.picking', pick_id,
                'button_done', cr)
        wave_obj.write(cr, uid, [wpick.wave_id.id], {'state': 'done'})
    return {
        'wave_picks_remaining': str(wpicks_remaining),
    }

def _get_default(self, cr, uid, data, context):
    if data['form']['wave_picks_remaining']:
        data['form']['wave_pick_notification'] = _('%s pickings left in the wave.') % (tools.ustr(data['form']['wave_picks_remaining']),)
    return data['form']

class partial_picking(wizard.interface):

    states = {
        'init': {
            'actions': [ _get_moves ],
            'result': {'type': 'form', 'arch': _moves_arch, 'fields': _moves_fields,
                'state' : (
                    ('end', 'Cancel'),
                    ('split', 'Make Picking')
                )
            },
        },
        'split': {
            'actions': [ _do_split ],
            'result': {'type': 'state', 'state': 'end2'},
        },
        'end2': {
            'actions': [ _get_default ],
            'result': {'type': 'form', 'arch': _moves_arch_end,
                'fields': _moves_fields_end,
                'state': (
                    ('end', 'Close'),
                )
            },
        },
    }

partial_picking('wave_partial_picking')
